# Problem 2 #

Recall that in lecture, we talked about the function declaration/definition system
and how its origins are rooted in the fact that computers were fairly weak back
when C++ was invented.

Another effect of this restriction was that you couldn't compile a file that was
too big, because you'd run out of memory trying to read it. To solve this, the 
creators of C created a system whereby smaller files could be compiled 
one at a time and then linked together. This is still the *only* real way to
compile sufficiently large C++ projects, and is something we'll be using heavily
for our projects throughout the semester.

For this problem, you will explore the basics of multistage compilation, along
with some of the common issues that come with it. This can be a little
complicated to wrap your head around, so [here's a StackOverflow post with more
information about this process]
(https://stackoverflow.com/questions/6264249/how-does-the-compilation-linking-process-work)
if you want it.

## Compiling a single file

When you invoke `g++` at the command line, you're actually invoking an 'overseer'
program (referred to by the compiler folks as a 'frontend driver'). This overseer
is responsible for running the many stages of compilation (which we'll learn 
about next week) and making sure that the data that's needed is being passed
around correctly.

What you need to know for this assignment is that to compile a single file, you
run the following command:

```
g++ <input>.cpp -c -o <output>.o
```

The `.o` file that gets generated contains binary code (basically, CPU instructions)
to run each of the functions *defined* within it. This type of file is known by
many names, of which "object file" is the most common, so that's what I'll
use.

Finally, note that while you can change the name of `<output>`, it's usually
best to keep the file name prefix, so a file named `food.cpp` gets compiled
to `food.o`, a file named `coffee.cpp` gets compiled to `coffee.o`, etc.
etc.

## Linking

Recall that we can compile code as long as we have a function declaration. This
means that we can compile a program for which some function definitions are
missing. What's left in it's place is called a fixup, which basically
says "call the function from here, once you have the code for the function."

For example, if we have a function that computes some complex numbers using sine
and cosine, the object file might look like:

```
complex_func:
   addq %rax, %rbx
   imul %ebx, %ecx
   < fixup `sin`>
   subq %rax, %rbx
   < fixup `cos`>
   retq
```

In this code, there are unresolved fixups for the `sin` and `cos` functions--the
compiler did not know where those functions were, so it just left placeholders.

Of course, in order to get a final, working program, we need to have all the
definitions. This is the job of the *linker*: it takes a bunch of object files
and resolves all the fixups, so that every function call works. If the linker
can't do this (e.g. because you don't actually have a definition for a function),
it'll error.

To invoke the linker, you need all your object files. Then, you can link them
by invoking `g++` once again:

```
g++ <obj1>.o <obj2>.o ... <objN>.o -o executable
```

This executable can then be run by the system.

## Question 0 ##

Execute `g++ -Wall -Wextra -c file.cpp -o file.o` for each `file.cpp` in this
directory. Make a note of the compilation errors that show up.

<div class=question><!-- P2, Q0 -->

**Pick one file (say which one!) and explain what every error/warning means. 
You will need to include the text of the error(s).**
</div>
<div class=answer>
<!-- FIXME: Your answer goes here-->
</div>

## Question 1 ##

As a general rule, we need to be able to compile each file individually before
we can make an executable program. 

<div class=question><!-- P2, Q1 -->

**Fix these programs so that they compile with no warnings/errors under `-Wall
-Wextra`.** You may *not* add additional functions, change the definitions of 
any existing functions, or delete any code.
</div>
<div class=answer>
<!-- FIXME: No answer is needed, but make sure you fix the files-->
</div>

## Question 2 ##

Now that we have finished compiling each file individually, we are ready to link
them. Run `g++ file1.o file2.o main.o` once you get all the files compiling
individually.

If you've followed the instructions so far, you'll get an error. Find out what
this error means (you're free to use google!)

<div class=question><!-- P2, Q2 -->

**Explain what this error means**.
</div>
<div class=answer>
<!-- FIXME: Your answer goes here-->
</div>

## Question 4 ##


<div class=question><!-- P2, Q3 -->

**Fix the error from the previous question. You will have to delete some code--this
is fine.**
</div>
<div class=answer>
<!-- FIXME: No answer is needed, but make sure you fix the files-->
</div>
