#!/usr/bin/env python3

import re
import copy
import os
import subprocess
from subprocess import CalledProcessError

answerFiles = ["problem0/problem0.md",
               "problem1/problem1.md",
               "problem2/problem2.md"]

cppFiles = [["problem0/problem0.cpp"],
            ["problem1/problem1.cpp"],
           ["problem2/file0.cpp","problem2/file1.cpp","problem2/main.cpp"]]

fixpat = re.compile("FIXME")

def checkAnswerFile(fname):
    haveWarn = False
    with open(fname,'r') as infile:
        for (lineno, line) in enumerate(infile):
            match = re.search(fixpat,line)
            if match:
                haveWarn = True
                print("You appear to have left a FIXME in " + fname + " at line "
                      + str(lineno) + ". Make sure you've answered that problem"
                      + " and delete the FIXME.")
    if not haveWarn:
        print(fname + " seems to be fine.")

    return haveWarn

def checkCompile(fileList):
    infiles = fileList
    objfiles = [ f[:-3] + "o" for f in infiles ]
    executable = os.path.join(os.path.dirname(fileList[0]),os.path.dirname(fileList[0]))
    haveWarn = False
    haveErr = False

    for (inf,outf) in zip(infiles,objfiles):
        try:
            output = subprocess.check_output(["g++",inf,"-Wall","-Wextra","-c","-o",outf],
                                         stderr=subprocess.PIPE)
            if output:
                haveWarn = True
                print("File " + inf + " generated output when compiling. Make sure "
                      + " it compiles without warnings under -Wall -Wextra.")
        except CalledProcessError:
            haveErr = True
            print("File " + inf + " errored out during compilation. This is " +
                  "definitely incorrect--check it for errors.")
    try:
        commandArray = ["g++"] + objfiles + ["-o",executable]
        print(" ".join(commandArray))
        output = subprocess.check_output(commandArray,
                                         stderr=subprocess.PIPE)
        if output:
            haveWarn = True
            print("Output generated when linking. Check for warnings/errors.")
    except:
            haveWarn = True
            print(executable + " errored during linking. This is not correct: " 
                 + "check for linker errors!")
    return haveWarn

haveWarn = False
for f in answerFiles:
    haveWarn = checkAnswerFile(f) or haveWarn
if not haveWarn:
    print("Answer files seem to be clean.")

for fl in cppFiles:
    haveWarn = checkCompile(fl) or haveWarn

if not haveWarn:
    print("Code files seem to compile fine.")

if haveWarn:
    print("At least some of your files seem to have issues.")
else:
    print("Your files pass the sanity check--make sure you look over them " +
           "yourself and make sure they look okay!")